
# q-expansion of newform 54.7.b.b, downloaded from the LMFDB on 19 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 54
weight = 7
poly_data = [2, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 2], 1]]

hecke_ring_character_values = [[29, [-1, 0]]]
aps_data = [[0, 2], [0, 0], [0, -15], [389, 0], [0, 735], [1415, 0], [0, 837], [-3067, 0], [0, 7401], [0, 4578], [-11338, 0], [47135, 0], [0, 2190], [145118, 0], [0, -63705], [0, -93978], [0, 127893], [-350305, 0], [120341, 0], [0, -118476], [175151, 0], [-252259, 0], [0, 119886], [0, 279297], [-1297105, 0], [0, -266160], [-1573315, 0], [0, -259299], [-485998, 0], [0, -438315], [2530766, 0], [0, -1072014], [0, -238989], [4010357, 0], [0, -993372], [1217045, 0], [-2814910, 0], [6925493, 0], [0, -47247], [0, 2998374], [0, 936324], [-2072257, 0], [0, -874263], [-987577, 0], [0, -3488391], [9556877, 0], [-4246819, 0], [11418374, 0], [0, -3124848], [3772010, 0], [0, -3938148], [0, -1505076], [-19802329, 0], [0, -1930986], [0, 9532506], [0, 1269492], [0, 1571841], [-1613491, 0], [24497714, 0], [0, -6407958], [-15958834, 0], [0, 6379509], [-15402418, 0], [0, -3261057], [-32781121, 0], [0, -433182], [13608677, 0], [60111983, 0], [0, -25080396], [32311343, 0], [0, 21132780], [0, 13777227], [-33888571, 0], [80664239, 0], [76388933, 0], [0, -3315696], [0, -882561], [-31229710, 0], [0, -16884888], [-41703289, 0], [0, 35514633], [74714303, 0], [0, 3066543], [-17435302, 0], [-88762858, 0], [0, 44963394], [0, -55424511], [-63096118, 0], [0, -41013201], [-133440739, 0], [0, -23249727], [0, -62190462], [57220733, 0], [0, -4529013], [-69988426, 0], [0, -64118889], [0, -13169121], [0, -42434217], [202394621, 0], [168172463, 0], [-97089091, 0], [0, -76373631], [0, 42966246], [0, 84992703], [-134707435, 0], [-336318073, 0], [0, -74297001], [0, 105777702], [0, 88251546], [-146561542, 0], [-103418971, 0], [144725159, 0], [0, -99829347], [240607901, 0], [367661045, 0], [0, -42973068], [135204446, 0], [0, 33213402], [0, -79883718], [0, -76384038], [-10377529, 0], [-61177921, 0], [0, 105564975], [0, 169863444], [264962846, 0], [0, -126012447], [-219800761, 0], [0, -34370370], [-150070570, 0], [287021234, 0], [419240990, 0], [0, 248891475], [621902549, 0], [46204967, 0], [0, 54622461], [-817868113, 0], [0, -48601314], [485178149, 0], [0, 350590404], [0, 259404732], [635498894, 0], [0, 16499337], [-362586931, 0], [0, -137541699], [496962551, 0], [0, -350328450], [263394119, 0], [0, -374053152], [-401087707, 0], [0, -99449541], [556535495, 0], [0, -291989025], [-538395379, 0], [0, 257686806], [-434990275, 0], [0, 15620730], [428965958, 0], [0, 225212856], [11123615, 0], [0, -278395053], [0, -74379891], [0, -444820851], [-812668867, 0], [0, 469166067], [0, 450295626], [0, 47621805], [-629069515, 0], [-993161230, 0]]
